<?php

class WaashipUserShippingMethodController
{
    function __construct()
    {
    }
}

add_action('woocommerce_checkout_update_order_meta', 'custom_meta_to_order', 20, 1);
function custom_meta_to_order($order_id)
{
    $order = wc_get_order($order_id);
    $shipping_method = $order->get_shipping_method();
    if (in_array($shipping_method, ['Waaship 全家超商配送', 'Waaship 7-11超商配送', 'Waaship 萊爾富超商配送'])) {
        $waaship_cvs_store_id = isset($_COOKIE['waaship_cvs_store_id']) ? sanitize_text_field($_COOKIE['waaship_cvs_store_id']) : '';
        $waaship_cvs_store_name = isset($_COOKIE['waaship_cvs_store_name']) ? sanitize_text_field($_COOKIE['waaship_cvs_store_name']) : '';
        $waaship_cvs_store_address = isset($_COOKIE['waaship_cvs_store_address']) ? sanitize_text_field($_COOKIE['waaship_cvs_store_address']) : '';
        $order->update_meta_data('waaship_cvs_store_id', $waaship_cvs_store_id);
        $order->update_meta_data('waaship_cvs_store_name', $waaship_cvs_store_name);
        $order->update_meta_data('waaship_cvs_store_address', $waaship_cvs_store_address);
        $order->save();
        unsetCVSData();
    }
}

add_action('woocommerce_review_order_after_shipping', 'shipping_choose_cvs');
function shipping_choose_cvs()
{
    $chosen_shipping = wc_get_chosen_shipping_method_ids();
    $host = get_site_url();
    $data = [];
    if (count($chosen_shipping) === 0) {
        unsetCVSData();
        return;
    }
    if (
        (isset($_COOKIE['waaship_cvs_open']) && $_COOKIE['waaship_cvs_open'] === 'family_mart' && $chosen_shipping[0] === 'waaship_family_mart') ||
        (isset($_COOKIE['waaship_cvs_open']) && $_COOKIE['waaship_cvs_open'] === 'seven_eleven' && $chosen_shipping[0] === 'waaship_seven_eleven') ||
        (isset($_COOKIE['waaship_cvs_open']) && $_COOKIE['waaship_cvs_open'] === 'hi_life' && $chosen_shipping[0] === 'waaship_hi_life')
    ) {
        $waaship_cvs_store_id = isset($_COOKIE['waaship_cvs_store_id']) ? sanitize_text_field($_COOKIE['waaship_cvs_store_id']) : '';
        $waaship_cvs_store_name = isset($_COOKIE['waaship_cvs_store_name']) ? sanitize_text_field($_COOKIE['waaship_cvs_store_name']) : '';
        $waaship_cvs_store_address = isset($_COOKIE['waaship_cvs_store_address']) ? sanitize_text_field($_COOKIE['waaship_cvs_store_address']) : '';
        $data = [
            'waaship_cvs_store_id'      => $waaship_cvs_store_id,
            'waaship_cvs_store_name'    => $waaship_cvs_store_name,
            'waaship_cvs_store_address' => $waaship_cvs_store_address
        ];
    } else {
        unsetCVSData();
    }
    if ($chosen_shipping[0] === 'waaship_seven_eleven') {
        wc_get_template('cart/chooseSevenCVS.php', array_merge([
            'callback_url' => $host . '/wp-json/waaship/cvs-choose-callback/seven-eleven',
            'protocol' => 'https://'
        ], $data), '', WAASHIP_PLUGIN_DIR . 'templates/');
        setcookie("waaship_cvs_open", 'seven_eleven', time() + 3600, '/');
    }
    if ($chosen_shipping[0] === 'waaship_family_mart') {
        wc_get_template('cart/chooseFamilyMartCVS.php', array_merge([
            'callback_url' => $host . '/wp-json/waaship/cvs-choose-callback/family-mart',
            'protocol' => 'https://'
        ], $data), '', WAASHIP_PLUGIN_DIR . 'templates/');
        setcookie("waaship_cvs_open", 'family_mart', time() + 3600, '/');
    }
    if ($chosen_shipping[0] === 'waaship_hi_life') {
        wc_get_template('cart/chooseHiLifeCVS.php', array_merge([
            'callback_url' => $host . '/wp-json/waaship/cvs-choose-callback/hi-life',
            'protocol' => 'https://'
        ], $data), '', WAASHIP_PLUGIN_DIR . 'templates/');
        setcookie("waaship_cvs_open", 'hi_life', time() + 3600, '/');
    }
}

function unsetCVSData()
{
    setcookie("waaship_cvs_store_id", "", time() - 3600);
    setcookie("waaship_cvs_store_name", "", time() - 3600);
    setcookie("waaship_cvs_store_address", "", time() - 3600);
    setcookie("waaship_cvs_open", "", time() - 3600);
}

add_filter('woocommerce_package_rates', 'packageRate', 10, 2);

function packageRate($rates)
{
    global $woocommerce;

    $total = $woocommerce->cart->cart_contents_total;
    $fee_logistics_data = []; // key => [enabled, price]
    $weight = $woocommerce->cart->get_cart_contents_weight();
    $waaship_cross_border_cost = 0;
    $shipping_method_list = $woocommerce->shipping->get_shipping_methods();
    foreach ($shipping_method_list as $method_id => $values) {
        if ('waaship_cross_border' === $values->id && $waaship_cross_border_cost == 0) {
            $title = 'range' . ((int)$weight - 1);
            $method_detail = $values->instance_settings;
            $waaship_cross_border_cost = $method_detail[$title];
        }
        if (!array_key_exists($values->id, $fee_logistics_data)) {
            $fee_logistics_data[$values->id] = [
                'enabled_free_logistics' => $values->instance_settings['enabled_free_logistics'] ?? 'no',
                'free_logistics_price'   => $values->instance_settings['free_logistics_price'] ?? 9999999
            ];
        }
    }
    foreach ($rates as $key => $rate) {
        if ('waaship_cross_border' === $rate->id) {
            $rates[$key]->cost = $waaship_cross_border_cost;
            if ($waaship_cross_border_cost == 0) {
                unset($rates[$key]);
            }
        }

        if ('waaship_family_mart' === $rate->id && $weight > 5) {
            unset($rates[$key]);
        }

        if ('waaship_seven_eleven' === $rate->id && $weight > 5) {
            unset($rates[$key]);
        }

        if ('waaship_hi_life' === $rate->id && $weight > 5) {
            unset($rates[$key]);
        }

        if ($fee_logistics_data[$rate->id]['enabled_free_logistics'] === 'yes' && $total >= $fee_logistics_data[$rate->id]['free_logistics_price']) {
            $rates[$key]->cost = 0;
        }
    }
    return $rates;
}


add_action('woocommerce_checkout_process', 'checkout_field_process');

function checkout_field_process()
{
    global $woocommerce;
    $shipping_method = $woocommerce->session->get('chosen_shipping_methods');
    if (count($shipping_method) === 0) {
        return;
    }
    if (in_array($shipping_method[0], ['waaship_family_mart', 'waaship_seven_eleven', 'waaship_hi_life', 'waaship_ok'])) {
        $waaship_cvs_store_id = isset($_COOKIE['waaship_cvs_store_id']) ? sanitize_text_field($_COOKIE['waaship_cvs_store_id']) : '';
        $waaship_cvs_store_name = isset($_COOKIE['waaship_cvs_store_name']) ? sanitize_text_field($_COOKIE['waaship_cvs_store_name']) : '';
        $waaship_cvs_store_address = isset($_COOKIE['waaship_cvs_store_address']) ? sanitize_text_field($_COOKIE['waaship_cvs_store_address']) : '';
        if (!$waaship_cvs_store_id || !$waaship_cvs_store_name || !$waaship_cvs_store_address) {
            wc_add_notice('請選擇取貨門市', 'error');
        }

        $shipping_name = sanitize_text_field($_POST['billing_last_name']) . sanitize_text_field($_POST['billing_first_name']);
        $shipping_phone = sanitize_text_field($_POST['billing_phone']);
        if ($_POST['ship_to_different_address'] == '1') {
            $shipping_name = sanitize_text_field($_POST['shipping_last_name']) . sanitize_text_field($_POST['shipping_first_name']);
        }
        if ((!preg_match('/^([\x{4e00}-\x{9fff}\x{3400}-\x{4dbf}]{2,5}|[a-zA-Z]{4,10})$/u', $shipping_name))) {
            wc_add_notice('收件人姓名必須為中文2~5個字或英文4~10個字', 'error');
        }

        if (!preg_match('/^09[0-9]{8}$/', $shipping_phone)) {
            wc_add_notice('收件人電話請輸入手機號碼共10碼', 'error');
        }
    }
}

new WaashipUserShippingMethodController();